searchData={"content_type":"text/markdown","items":[{"doc":"A `Trunk` is a module used to handle the transformation and storage of files with multiple versions.","ref":"Trunk.html","title":"Trunk","type":"behaviour"},{"doc":"To generate a `Trunk` call `use Trunk` and supply options to configure the specific behaviour you need.\n```\ndefmodule MyTrunk do\n  use Trunk, versions: [:original],\n             storage: Trunk.Storage.Filesystem,\n             storage_opts: [path: \"/tmp\"]\nend\n```","ref":"Trunk.html#module-usage","title":"Usage: - Trunk","type":"behaviour"},{"doc":"Options can be set at multiple levels\n- Global, use `config :trunk, key: :value`\n- Per otp app, use `config :my_app, trunk: [key: :value]`\n- Per module, supply options to `use Trunk`. Use `use Trunk, otp_app: :my_app` to read in per otp app options.\n- Per function, you can supply additional or override options to every function call.\n\n#","ref":"Trunk.html#module-options","title":"Options - Trunk","type":"behaviour"},{"doc":"- `:versions` a list of versions as atoms, `[:original, :thumb]`\n- `:async` (boolean) default `true`, whether to process each version in parallel or in sequence.\n- `:timeout` default `5_000`, how long to wait for each versions transformation and storage to complete when processing in parallel. **Note: ** if storing files non-locally (i.e. Amazon S3), then include both expected time for transformation and expected time for storage in this timeout.\n- `:storage` default `Trunk.Storage.Filesystem`, the storage module to use when storing files and versions\n- `:storage_opts` default: `[path: \"\"]`, the options for the storage module. See each storage module's documentation for available options.\n- `:otp_app`, only used at module level to read options specific to the otp app.\n\n#","ref":"Trunk.html#module-available-options","title":"Available options - Trunk","type":"behaviour"},{"doc":"Any extra options passed in at the otp app, module or function level are made available in the `opts` field of `Trunk.State`. This means that you can pass in any options you might need to use in any of the processing steps.","ref":"Trunk.html#module-additional-options","title":"Additional options - Trunk","type":"behaviour"},{"doc":"When files are stored, they are passed through a transformation pipeline which allows you to generate different versions of a file.\nYou might get a photo uploaded which you want to save along with a thumbnail. You might have a video which also needs a thumbnail extracted. You could convert a spreadsheet to CSV or a document to PDF.\n\nAt each point in the pipeline you have access to a `Trunk.State` or `Trunk.VersionState` struct which contains information collected at each step. Each state struct has an assigns map (similar to `Plug.Conn`) in which you can store information to be used later in the pipeline process or once storage is complete.\n\nThe storage pipeline goes through the following steps:\n- `c:preprocess/1` - Here you have access to the information about the file including the path to the file on disk. This is where you can validate the file or extract information that can be useful for later processing. This callback is called only once for the pipeline.\n- `c:transform/2` - This is where you determine how each version should be transformed. You can return a transformation instruction or a function that will do the actual transformation. This callback is called once per version.\n- `c:postprocess/3` - Here you have access to information about the version after its transformation. At this point you have access to the state about the version including the path to the temporary file on disk. You can extract information about the version file like file size or hash its contents. This callback is called once per version.\n- `c:storage_dir/2` - This is where you determine at which path the version should be saved. This callback is called once per version.\n- `c:filename/2` - This is where you determine which filename the version should be saved as. This callback is called once per version.\n- `c:storage_opts/2` - This allows you to set additional storage options like file permissions `:acl` or add things like S3 headers (Check with the storage module as to what options are avialble to be set here). This callback is called once per version.\n- The file is saved using the configured storage module.\n\n#","ref":"Trunk.html#module-file-storage","title":"File storage - Trunk","type":"behaviour"},{"doc":"```\ndefmodule MyTrunk do\n  use Trunk, versions: [:original, :thumb],\n             storage: Trunk.Storage.Filesystem,\n             storage_opts: [path: \"/tmp\"]\n\n  # Ensure the file is a JPG or PNG file\n  def preprocess(%Trunk.State{} = state) do\n    if String.downcase(extname) in [\".png\", \".jpg\", \".jpeg\"] do\n      {:ok, state}\n    else\n      {:error, \"Invalid file\"}\n    end\n  end\n\n  # Do not transform the original file\n  def transform(%Trunk.State{}, :original), do: nil\n  # Resize the file to a thumbnail of maximum 200x200px\n  def transform(%Trunk.State{}, :thumb),\n    do: {:convert, \"-strip -thumbnail 200x200> -limit area 10MB -limit disk 100MB\"}\n\n  # Store the file size of each version\n  def postprocess(%Trunk.VersionState{temp_path: temp_path} = version_state, _version, %Trunk.State{} = _state) do\n    %File.State{size: file_size} = File.stat(temp_path)\n    {:ok, Trunk.VersionState.assign(version_state, :file_size, file_size)}\n  end\n\n  # Store all versions in a directory based on a model id\n  def storage_dir(%Trunk.State{scope: %{id: model_id}}, _version),\n    do: to_string(model_id)\n\n  # Store the original file with its original filename\n  def filename(%Trunk.State{filename: filename}, :original),\n    do: filename\n  # Store other versions with the version in its filename\n  def filename(%Trunk.State{rootname: rootname, extname: extname}, version),\n    do: \"#{rootname}_#{version}#{extname}\"\n\n  def storage_opts(%Trunk.State{}, :original),\n    do: [acl: 0o600]\n  def storage_opts(%Trunk.State{}, :thumb),\n    do: [acl: 0o644]\nend\n\n> {:ok, %Trunk.State{filename: filename, versions: versions}} = MyTrunk.store(\"/path/to/photo.jpg\")\n> filename\n\"photo.jpg\"\n> versions |> Enum.map(fn({version, %Trunk.VersionState{assigns: %{file_size: file_size}}}) -> {version, file_size} end)\n[original: 34567, thumb: 456]\n```","ref":"Trunk.html#module-example","title":"Example: - Trunk","type":"behaviour"},{"doc":"Copies the stored file(s) from the `storage_dir/2` and `filename/2` of the original scope to the same of the new scope. **This function is generated by `use Trunk`**\n\nIt returns an `{:ok, %Trunk.State{}}` tuple of the new state on success and `{:error, %Trunk.State{}}` tuple of the new state if an error occurred anywhere in the processing pipeline.\n\n- `file_info` - The file name or base info needed to identify the file. Minimum is `%{filename: \"file.ext\"}`\n- `scope` - a map or struct that will help when generating the filename and storage directory for retrieving the file(s)\n- `new_scope` - a map or struct that will help when generating the filename and storage directory for saving the file(s)\n- `opts` - (optional) options to override module, app, or global options. See \"Options\" in the module documentations for all options.","ref":"Trunk.html#c:copy/4","title":"Trunk.copy/4","type":"callback"},{"doc":"Deletes the identified file and all its versions. **This function is generated by `use Trunk`**\n\nCalls `c:delete/3` with `delete(file_info, nil, [])`","ref":"Trunk.html#c:delete/1","title":"Trunk.delete/1","type":"callback"},{"doc":"Deletes the identified file and all its versions. **This function is generated by `use Trunk`**\n\n- With a `scope`, calls `c:delete/3` with `delete(file_info, scope, [])`\n- With `opts`, calls `c:delete/3` with `delete(file_info, nil, opts)`","ref":"Trunk.html#c:delete/2","title":"Trunk.delete/2","type":"callback"},{"doc":"Deletes the identified file and all its versions. **This function is generated by `use Trunk`**\n\nIt returns an `{:ok, %Trunk.State{}}` tuple on success and `{:error, %Trunk.State{}}` tuple if an error occurred.\n\n- `file_info` - The file name or base info needed to identify the file. Minimum is `%{filename: \"file.ext\"}`\n- `scope` - (optional) a map or struct that will help when generating the filename and storage directory for saving the file\n- `opts` - (optional) options to override module, app, or global options. See \"Options\" in the module documentations for all options.","ref":"Trunk.html#c:delete/3","title":"Trunk.delete/3","type":"callback"},{"doc":"A callback that should be used to generate the filename specific to each version.\nReturn `nil` to prevent storing a specific version\n\n- `state` - The trunk state\n- `version` - An atom representing the version\n\nExample:\n```\n# For the :original version return the original file name\ndef filename(%{filename: filename}, :original), do: filename\n# For any other version append the version to the root name\n#   mypic.jpg with :thumb version becomes mypic_thumb.jpg\ndef filename(%{rootname: rootname, extname: extname}, version),\n  do: \"#{rootname}_#{version}#{extname}\"\n```","ref":"Trunk.html#c:filename/2","title":"Trunk.filename/2","type":"callback"},{"doc":"A callback that can be used to do additional processing on each version file before it gets saved.\n\nThis callback can update the version state information which can then be used in `storage_dir/2` and `filename/2` as additional information for storage.\n\n**Note: ** If information is stored in the version state and used for file naming, then that information needs to be stored in order to retrieve that file or generate a URL etc.","ref":"Trunk.html#c:postprocess/3","title":"Trunk.postprocess/3","type":"callback"},{"doc":"To generate and store the md5 hash of each version.\n```\ndef postprocess(%Trunk.VersionState{temp_path: temp_path} = version_state, _version, _state) do\n  hash = :crypto.hash(:md5, File.read!(temp_path)) |> Base.encode16(case: :lower)\n  {:ok, Trunk.VersionState.assign(version_state, :hash, hash)}\nend\n```","ref":"Trunk.html#c:postprocess/3-example","title":"Example: - Trunk.postprocess/3","type":"callback"},{"doc":"To calculate and store the file size of each version.\n```\ndef postprocess(%Trunk.VersionState{temp_path: temp_path} = version_state, _version, _state) do\n  {:ok, %File.Stat{size: file_size}} = File.stat(temp_path)\n  {:ok, Trunk.VersionState.assign(version_state, :file_size, file_size)}\nend\n```","ref":"Trunk.html#c:postprocess/3-example","title":"Example: - Trunk.postprocess/3","type":"callback"},{"doc":"A callback that can be used to do any preprocessing on the state before transformation and storage begins.\n\nThis is a good place to do file validation (**Note** this example is wrapped in a convenience macro `validate_file_extensions/1`.","ref":"Trunk.html#c:preprocess/1","title":"Trunk.preprocess/1","type":"callback"},{"doc":"```\ndef preprocess(%Trunk.State{lower_extname: extname} = state) do\n  if extname in [\".jpg\", \".png\"] do\n    {:ok, state}\n  else\n    {:error, \"Invalid file\"}\n  end\nend\n```\n\nThis is also a place to do any processing that might be needed when transforming versions.","ref":"Trunk.html#c:preprocess/1-example","title":"Example - Trunk.preprocess/1","type":"callback"},{"doc":"Reprocesses the original file for all versions. **This function is generated by `use Trunk`**\n\nCalls `c:reprocess/4` with `reprocess(file_info, nil, :all, [])`","ref":"Trunk.html#c:reprocess/1","title":"Trunk.reprocess/1","type":"callback"},{"doc":"Reprocesses the original file for all or the specified version(s). **This function is generated by `use Trunk`**\n\n- With a `scope`, calls `c:reprocess/4` with `reprocess(file_info, scope, :all, [])`\n- With `opts`, calls `c:reprocess/4` with `reprocess(file_info, nil, :all, opts)`\n- With `version`, calls `c:reprocess/4` with `reprocess(file_info, nil, version, [])`","ref":"Trunk.html#c:reprocess/2","title":"Trunk.reprocess/2","type":"callback"},{"doc":"Reprocesses the original file for all or the specified version(s). **This function is generated by `use Trunk`**\n\n- With version(s) and opts, calls `c:reprocess/4` with `reprocess(file_info, nil, version, opts)`\n- With a scope and version(s), calls `c:reprocess/4` with `reprocess(file_info, scope, version, [])`\n- With a scope and opts, calls `c:reprocess/4` with `reprocess(file_info, scope, :all, opts)`","ref":"Trunk.html#c:reprocess/3","title":"Trunk.reprocess/3","type":"callback"},{"doc":"Reprocesses the original file for the specified version(s). **This function is generated by `use Trunk`**\n\nIt returns an `{:ok, path}` tuple on success and `{:error, error}` tuple if an error occurred.\n\n- `file_info` - The file name or base info needed to identify the file. Minimum is `%{filename: \"file.ext\"}`\n- `scope` - (optional) a map or struct that will help when generating the filename and storage directory for saving the file\n- `version` - (optional) an atom representing the version\n- `opts` - (optional) options to override module, app, or global options. See \"Options\" in the module documentations for all options.","ref":"Trunk.html#c:reprocess/4","title":"Trunk.reprocess/4","type":"callback"},{"doc":"Retrieves the original for the identified file. **This function is generated by `use Trunk`**\n\nCalls `c:retrieve/4` with `retrieve(file_info, nil, :original, [])`","ref":"Trunk.html#c:retrieve/1","title":"Trunk.retrieve/1","type":"callback"},{"doc":"Retrieves the original, or specified version, for the identified file. **This function is generated by `use Trunk`**\n\n- With a `scope`, calls `c:retrieve/4` with `retrieve(file_info, scope, :original, [])`\n- With `opts`, calls `c:retrieve/4` with `retrieve(file_info, nil, :original, opts)`\n- With `version`, calls `c:retrieve/4` with `retrieve(file_info, nil, version, [])`","ref":"Trunk.html#c:retrieve/2","title":"Trunk.retrieve/2","type":"callback"},{"doc":"Retrieves the original, or specified version, for the identified file. **This function is generated by `use Trunk`**\n\n- With a version and opts, calls `c:retrieve/4` with `retrieve(file_info, nil, version, opts)`\n- With a scope and version, calls `c:retrieve/4` with `retrieve(file_info, scope, version, [])`\n- With a scope and opts, calls `c:retrieve/4` with `retrieve(file_info, scope, :original, opts)`","ref":"Trunk.html#c:retrieve/3","title":"Trunk.retrieve/3","type":"callback"},{"doc":"Retrieves the version for the identified file. **This function is generated by `use Trunk`**\n\nIt returns an `{:ok, path}` tuple on success and `{:error, error}` tuple if an error occurred.\n\n- `file_info` - The file name or base info needed to identify the file. Minimum is `%{filename: \"file.ext\"}`\n- `scope` - (optional) a map or struct that will help when generating the filename and storage directory for saving the file\n- `version` - (optional) an atom representing the version\n- `opts` - (optional) options to override module, app, or global options. See \"Options\" in the module documentations for all options.\n           Additional options:\n             - `:output_path` a specific path (including filename) to save the file to. Defaults to a temporary file path","ref":"Trunk.html#c:retrieve/4","title":"Trunk.retrieve/4","type":"callback"},{"doc":"A callback that should be used to determine the storage directory specific to each version.\n\nEach storage system will have a base path in which to store files. The result of `storage_dir/2` is then appended to this to determine where to save files.\n\n- `state` - The trunk state\n- `version` - An atom representing the version\n\nExample:\n```\n# Assuming a base path of /tmp files will be saved in /tmp\ndef storage_dir(_state, _version), do: \"\"\n\n# Place the file in a directory named after a model id\n#  With a base path of /tmp and a scope of %{id: 42}, the file will be saved in /tmp/42/\ndef storage_dir(%Trunk.State{scope: %{id: model_id}}, _version),\n  do: to_string(model_id)\n```","ref":"Trunk.html#c:storage_dir/2","title":"Trunk.storage_dir/2","type":"callback"},{"doc":"A callback that should be used to set version specific storage options.\n\nThis is the place to set access permissions, storage headers (s3) and other options provided by the storage module.\n\nThis callback should return a list of options.\n\nExample:\n```\n# With Filesystem\ndef storage_opts(_state, _version), do: [acl: \"0600\"]\n\n# With S3\ndef storage_opts(_state, _version), do: [acl: :public_read]\n```","ref":"Trunk.html#c:storage_opts/2","title":"Trunk.storage_opts/2","type":"callback"},{"doc":"Stores the supplied file. **This function is generated by `use Trunk`**\n\nCalls `c:store/3` with `store(file, nil, [])`","ref":"Trunk.html#c:store/1","title":"Trunk.store/1","type":"callback"},{"doc":"Stores the supplied file. **This function is generated by `use Trunk`**\n\nCalls `c:store/3` with `store(file, nil, opts)`","ref":"Trunk.html#c:store/2","title":"Trunk.store/2","type":"callback"},{"doc":"Stores the supplied file after running it through the processing pipeline. **This function is generated by `use Trunk`**\n\nIt returns an `{:ok, %Trunk.State{}}` tuple on success and `{:error, %Trunk.State{}}` tuple if an error occurred anywhere in the processing pipeline.\n\n- `file` - the file to store.\n  - This can be a full path to file, or\n  - a url to a file, or\n  - a map with `:filename` and `:path` keys (e.g. `%Plug.Upload{}`), or\n  - a map with `:filename` and `:binary` keys (e.g. `%{filename: \"myfile.jpg\", binary:  >}`)\n- `scope` - (optional) a map or struct that will help when generating the filename and storage directory for saving the file\n- `opts` - (optional) options to override module, app, or global options. See \"Options\" in the module documentations for all options.","ref":"Trunk.html#c:store/3","title":"Trunk.store/3","type":"callback"},{"doc":"A callback that can be used to generate a transform specific to each version\n\nThe transformation instruction is a two element tuple `{command, arguments}`, a three element tuple `{command, arguments, extension}`, or a function that accepts a single argument (the source file path) and returns either `{:ok, \"/path/to/transformed/file\"}` or `{:error, \"Reason\"}`","ref":"Trunk.html#c:transform/2","title":"Trunk.transform/2","type":"callback"},{"doc":"To generate a thumnail, you might have a version named `:thumb`.\nA transform that runs the uploaded file through ImageMagick’s `convert` could be the following\n```\ndef transform(%Trunk.State{}, :thumb),\n  do: {:convert, \"-strip -thumbnail 200x200>\"}\n```\nThis would generate a call to `System.cmd` as follows:\n```\n# input_file is the source (uploaded) file\n# output_file is a temporary file with the same extension as the source file\nSystem.cmd \"convert\", [input_file, \"-strip\", \"-thumbnail\", \"200x200>\", output_file], stderr_to_stdout: true\n```\n\nTo generate a JPEG thumbnail whether the source is JPEG or PNG you would add the extension to the transform instruction.\n```\ndef transform(_state, :thumb),\n  do: {:convert, \"-strip -thumbnail 200x200>\", :jpg}\n```\nIn this case the temporary file generated for the transformation would have a .jpg extension.\n\nIn either form, the arguments can be generated by a function which receives the input and output file paths. This is useful for transforms where the input and output paths are not the first and last respectively.\n```\ndef transform(_state, _version),\n  do: {:convert, fn(input, output) -> [\"-density\", \"300\", input, \"-flatten\", \"-strip\", \"-thumbnail\", \"200x200>\", output] end, :jpg}\n```\n\n**Note:**  Returning an instruction with an extension will change the extension of the temporary file used in the transformation but will not affect the filename during save. That still needs to be done in `c:filename/2`\n\nFinally a function can be returned which will be wholly responsible for doing the transformation and returning the location of the transformed file.\nThe function should return `{:ok, file_path}` if successful, or `{:error, reason}` if not.\n\nThis example uses `soffice` to transform an XLS document to CSV. The problem is `soffice` doesn’t allow the output file to be specified so the above `{:soffice, [args]}` style can’t be used. To solve this, a transformation function is used to handle the transformation.\nThe function creates a temporary directory for the transformation, runs the `soffice` command and then parses the output for the resulting file.\n```\ndef transform(%Trunk.State{extname: extname}, :csv) when extname in [\".xls\", \".xlsx\"],\n  do: fn(source_path) ->\n        {:ok, directory} = Briefly.create(directory: true)\n        # soffice must be in the PATH\n        case System.cmd(\"soffice\", [\"--headless\", \"--convert-to\", \"csv:Text - txt - csv (StarCalc):44,34,76,0\", \"--outdir\", directory, source_path], stderr_to_stdout: true) do\n          {output, 0} ->\n            [_, path] = Regex.run(~r/-> (.+\\.csv)/, output)\n            {:ok, path}\n          {result, _} -> {:error, result}\n        end\n      end\ndef transform(_state, _version), do: nil\n```","ref":"Trunk.html#c:transform/2-example","title":"Example: - Trunk.transform/2","type":"callback"},{"doc":"Generates a URL for the given information. **This function is generated by `use Trunk`**\nWill return nil if the `filename/2` callback returns nil\n\nCalls `c:url/4` with `url(file_info, nil, :original, [])`","ref":"Trunk.html#c:url/1","title":"Trunk.url/1","type":"callback"},{"doc":"Generates a URL for the given information. **This function is generated by `use Trunk`**\nWill return nil if the `filename/2` callback returns nil\n\n- With a `version`, calls `c:url/4` with `url(file_info, nil, version, [])`\n- With a `scope`, calls `c:url/4` with `url(file_info, scope, :original, [])`\n- With `opts`, calls `c:url/4` with `url(file_info, nil, :original, opts)`","ref":"Trunk.html#c:url/2","title":"Trunk.url/2","type":"callback"},{"doc":"Generates a URL for the given information. **This function is generated by `use Trunk`**\nWill return nil if the `filename/2` callback returns nil\n\n- With a `version`, calls `c:url/4` with `url(file_info, nil, version, opts)`\n- With a `scope`, calls `c:url/4` with `url(file_info, scope, :original, opts)`","ref":"Trunk.html#c:url/3","title":"Trunk.url/3","type":"callback"},{"doc":"Generates a URL for the given information. **This function is generated by `use Trunk`**\nWill return nil if the `filename/2` callback returns nil\n\n- `file_info` - The file name or base info needed to identify the file. Minimum is `%{filename: \"file.ext\"}`\n- `scope` - The scope object\n- `version` - The file version to which the URL must point\n- `opts` - Override options.","ref":"Trunk.html#c:url/4","title":"Trunk.url/4","type":"callback"},{"doc":"A convenience macro for generating a `c:preprocess/1` function that validates the file being saved against a list of approved extensions (case-insensitive).","ref":"Trunk.html#validate_file_extensions/1","title":"Trunk.validate_file_extensions/1","type":"macro"},{"doc":"```\ndefmodule MyTrunk do\n  use Trunk, versions: [:original]\n\n  validate_file_extensions ~w[.jpg .jpeg .png]\nend\n```","ref":"Trunk.html#validate_file_extensions/1-example","title":"Example: - Trunk.validate_file_extensions/1","type":"macro"},{"doc":"","ref":"Trunk.html#t:args/0","title":"Trunk.args/0","type":"type"},{"doc":"","ref":"Trunk.html#t:command/0","title":"Trunk.command/0","type":"type"},{"doc":"","ref":"Trunk.html#t:ext/0","title":"Trunk.ext/0","type":"type"},{"doc":"","ref":"Trunk.html#t:file/0","title":"Trunk.file/0","type":"type"},{"doc":"","ref":"Trunk.html#t:file_info/0","title":"Trunk.file_info/0","type":"type"},{"doc":"","ref":"Trunk.html#t:new_scope/0","title":"Trunk.new_scope/0","type":"type"},{"doc":"","ref":"Trunk.html#t:opts/0","title":"Trunk.opts/0","type":"type"},{"doc":"","ref":"Trunk.html#t:reason/0","title":"Trunk.reason/0","type":"type"},{"doc":"","ref":"Trunk.html#t:scope/0","title":"Trunk.scope/0","type":"type"},{"doc":"","ref":"Trunk.html#t:storage_opts/0","title":"Trunk.storage_opts/0","type":"type"},{"doc":"","ref":"Trunk.html#t:transform_func/0","title":"Trunk.transform_func/0","type":"type"},{"doc":"","ref":"Trunk.html#t:version/0","title":"Trunk.version/0","type":"type"},{"doc":"","ref":"Trunk.html#t:versions/0","title":"Trunk.versions/0","type":"type"},{"doc":"This module defines a `Trunk.State` struct and provides some helper functions for working with that state.","ref":"Trunk.State.html","title":"Trunk.State","type":"module"},{"doc":"The following fields are available in the state object. Some values are filled in during processing.\n- `filename` - The base filename of the file being processed. (e.g. `\"photo.jpg\"`)\n- `rootname` - The root of the filen being processed. (e.g. `\"photo\"`)\n- `extname` - The file extension of the file being processed. (e.g. `\".jpg\"`)\n- `lower_extname` - The file extension of the file being processed forced to lower case (e.g. `\"*.jpg\"`, even if the file is `\"PHOTO.JPG\"`)\n- `path` - The full path to the file being processed. If the file was passed in as a binary, it is a path to the temporary file created with that binary.\n- `versions` - A map of the versions and their respective `Trunk.VersionState` (e.g. `%{original: %Trunk.VersionState{}, thumbnail: %Trunk.VersionState{}}`)\n- `scope` - A user struct/map passed in useful for determining storage locations and file naming.\n- `async` - A boolean indicator of whether processing will be done in parallel.\n- `timeout` - The timeout after which each processing process will be terminated. (Only applies with `async: true`)\n- `storage` - The module to use for storage processing. (e.g. `Trunk.Storage.Filesystem` or `Trunk.Storage.S3`)\n- `storage_opts` - A keyword list of options for the `storage` module\n- `errors` - a place to record errors encountered during processing. (`nli` if no errors, otherwise a map of errors)\n- `opts` - All the options merged together (see Options in `Trunk` module documentation).\n- `assigns` - shared user data as a map (Same as assigns in  `Plug.Conn`)","ref":"Trunk.State.html#module-fields","title":"Fields - Trunk.State","type":"module"},{"doc":"Assigns a value to a key on the state.","ref":"Trunk.State.html#assign/3","title":"Trunk.State.assign/3","type":"function"},{"doc":"```\niex> state.assigns[:hello]\nnil\niex> state = Trunk.State.assign(state, :hello, :world)\niex> state.assigns[:hello]\n:world\n```","ref":"Trunk.State.html#assign/3-example","title":"Example: - Trunk.State.assign/3","type":"function"},{"doc":"Retrieves an assign value for a specific version.","ref":"Trunk.State.html#get_version_assign/3","title":"Trunk.State.get_version_assign/3","type":"function"},{"doc":"```\niex> state = %Trunk.State{versions: %{thumbnail: %Trunk.VersionState{assigns: %{hello: :world}}}}\niex> %Trunk.State.get_version_assign(state, :thumbnail, :hello)\n:world\niex> %Trunk.State.get_version_assign(state, :thumbnail, :unknown)\nnil\n```","ref":"Trunk.State.html#get_version_assign/3-example","title":"Example: - Trunk.State.get_version_assign/3","type":"function"},{"doc":"","ref":"Trunk.State.html#init/3","title":"Trunk.State.init/3","type":"function"},{"doc":"Puts an error into the error map.","ref":"Trunk.State.html#put_error/4","title":"Trunk.State.put_error/4","type":"function"},{"doc":"```\niex> state.errors\nnil\niex> state = Trunk.State.put_error(state, :thumb, :transform, \"Error with convert blah blah\")\niex> state.errors\n%{thumb: [transform: \"Error with convert blah blah\"]}\n```","ref":"Trunk.State.html#put_error/4-example","title":"Example: - Trunk.State.put_error/4","type":"function"},{"doc":"Restore a saved state from a filename, JSON, or a map","ref":"Trunk.State.html#restore/2","title":"Trunk.State.restore/2","type":"function"},{"doc":"```\niex> Trunk.State.restore(\"photo.jpg\")\n%Trunk.State{filename: \"photo.jpg\"}\niex> Trunk.State.restore(%{filename: \"photo.jpg\", assigns: %{hash: \"abcdef\"}}\n%Trunk.State{filename: \"photo.jpg\", assigns: %{hash: \"abcdef\"}}\niex> Trunk.State.restore(%{\"filename\" => \"photo.jpg\", \"assigns\" => %{\"hash\" => \"abcdef\"}}\n%Trunk.State{filename: \"photo.jpg\", assigns: %{hash: \"abcdef\"}}\niex> Trunk.State.restore(\"{\\\"filename\\\": \\\"photo.jpg\\\", \\\"assigns\\\": {\\\"hash\\\": \\\"abcdef\\\"}}\")\n%Trunk.State{filename: \"photo.jpg\", assigns: %{hash: \"abcdef\"}}\n```","ref":"Trunk.State.html#restore/2-example","title":"Example: - Trunk.State.restore/2","type":"function"},{"doc":"Extracts the data needed from the state in order to reconstruct the file paths in future.\n\nOptions:\n- `:as` - How to save the state.\n  - `:string` - Default, will just save the file name. An error will be raised if there are any assigns unless `:ignore_assigns` is set to tru\n  - `:map` - will save a map with keys `:filename`, `:assigns`, and `:version_assigns`\n  - `:json` - will save a map encoded as JSON (Requires Poison library to be included in deps)\n- `:ignore_assigns` boolean, default false. Use this to save as string and ignore any assigns (Make sure you’re not using assigns for `c:Trunk.storage_dir/2` or `c:Trunk.filename/2`)\n- `:assigns` - a list of keys to save from the assigns hashes","ref":"Trunk.State.html#save/2","title":"Trunk.State.save/2","type":"function"},{"doc":"```\niex> Trunk.State.save(%Trunk.State{filename: \"photo.jpg\"})\n\"photo.jpg\"\niex> Trunk.State.save(%Trunk.State{filename: \"photo.jpg\", assigns: %{hash: \"abcdef\"}}, as: :map)\n%{filename: \"photo.jpg\", assigns: %{hash: \"abcdef\"}}\niex> Trunk.State.save(%Trunk.State{filename: \"photo.jpg\", assigns: %{hash: \"abcdef\", file_size: 12345}}, as: :map, assigns: [:hash])\n%{filename: \"photo.jpg\", assigns: %{hash: \"abcdef\"}}\niex> Trunk.State.save(%Trunk.State{filename: \"photo.jpg\", assigns: %{hash: \"abcdef\"}}, as: :json)\n\"{\\\"filename\\\": \\\"photo.jpg\\\", \\\"assigns\\\": {\\\"hash\\\": \\\"abcdef\\\"}}\"\n```","ref":"Trunk.State.html#save/2-example","title":"Example: - Trunk.State.save/2","type":"function"},{"doc":"","ref":"Trunk.State.html#t:assign_keys/0","title":"Trunk.State.assign_keys/0","type":"type"},{"doc":"","ref":"Trunk.State.html#t:file_info/0","title":"Trunk.State.file_info/0","type":"type"},{"doc":"","ref":"Trunk.State.html#t:opts/0","title":"Trunk.State.opts/0","type":"type"},{"doc":"","ref":"Trunk.State.html#t:save_opts/0","title":"Trunk.State.save_opts/0","type":"type"},{"doc":"","ref":"Trunk.State.html#t:t/0","title":"Trunk.State.t/0","type":"type"},{"doc":"","ref":"Trunk.State.html#t:version/0","title":"Trunk.State.version/0","type":"type"},{"doc":"This is a behaviour that can be implemented by any storage module to make sure it supports all required functions.","ref":"Trunk.Storage.html","title":"Trunk.Storage","type":"behaviour"},{"doc":"Generates a URL to the S3 object\n\n- `directory` - The directory\n- `filename` - The name of the file\n- `opts` - The options for the storage system, set in Trunk option `:storage_opts`","ref":"Trunk.Storage.html#c:build_uri/3","title":"Trunk.Storage.build_uri/3","type":"callback"},{"doc":"```\nMyStorage.build_url(\"path/to\", \"file.ext\", some: :opt)\n#=> \"https://my-storage/my-bucket/path/to/file.ext\"\n```","ref":"Trunk.Storage.html#c:build_uri/3-example","title":"Example: - Trunk.Storage.build_uri/3","type":"callback"},{"doc":"Deletes the version file from the storage system at path `Path.join(directory, filename)`\n\n- `directory` - The directory\n- `filename` - The name of the file\n- `opts` - The options for the storage system, set in Trunk option `:storage_opts`","ref":"Trunk.Storage.html#c:delete/3","title":"Trunk.Storage.delete/3","type":"callback"},{"doc":"The file should be removed from  /my-bucket/path/to/file.ext\n```\nMyStorage.delete(\"path/to/\", \"file.ext\", some: :opt)","ref":"Trunk.Storage.html#c:delete/3-example","title":"Example: - Trunk.Storage.delete/3","type":"callback"},{"doc":"","ref":"Trunk.Storage.html#c:retrieve/4","title":"Trunk.Storage.retrieve/4","type":"callback"},{"doc":"Save the `source_file` to the storage system on path `Path.join(directory, filename)`\n\n- `directory` - The directory\n- `filename` - The name of the file\n- `source_path` - The full path to the file to be stored. This is a path to the uploaded file or a temporary file that has undergone transformation\n- `opts` - The options for the storage system, set in Trunk option `:storage_opts`","ref":"Trunk.Storage.html#c:save/4","title":"Trunk.Storage.save/4","type":"callback"},{"doc":"The file should be saved to  /my-bucket/path/to/file.ext\n```\nMyStorage.save(\"path/to/\", \"file.ext\", \"/tmp/uploaded_file.ext\", some: :opt)","ref":"Trunk.Storage.html#c:save/4-example","title":"Example: - Trunk.Storage.save/4","type":"callback"},{"doc":"","ref":"Trunk.Storage.html#t:opts/0","title":"Trunk.Storage.opts/0","type":"type"},{"doc":"A `Trunk.Storage` implementation for the local file system.","ref":"Trunk.Storage.Filesystem.html","title":"Trunk.Storage.Filesystem","type":"module"},{"doc":"Generates a URL for the storage directory and file\n\n- `directory` - The relative directory where the file is saved\n- `filename` - The name of the file that was previous saved\n- `opts` - The options for the storage system\n  - `base_url:` (optional) The base url to join the path to","ref":"Trunk.Storage.Filesystem.html#build_uri/3","title":"Trunk.Storage.Filesystem.build_uri/3","type":"function"},{"doc":"```\nTrunk.Storage.Filesystem.build_url(\"path/to\", \"file.ext\")\n#=> \"path/to/file.ext\"\nTrunk.Storage.Filesystem.build_url(\"path/to\", \"file.ext\", base_uri: \"http://example.com\")\n#=> \"http://example.com/path/to/file.ext\"\nTrunk.Storage.Filesystem.build_url(\"path/to\", \"file.ext\", base_uri: \"http://example.com/uploads/\")\n#=> \"http://example.com/uploads/path/to/file.ext\"\nTrunk.Storage.Filesystem.build_url(\"path/to\", \"file.ext\", base_uri: \"/uploads/\")\n#=> \"/uploads/path/to/file.ext\"\n```","ref":"Trunk.Storage.Filesystem.html#build_uri/3-example","title":"Example: - Trunk.Storage.Filesystem.build_uri/3","type":"function"},{"doc":"Copies the file within the file system.\n\n- `directory` - The relative directory within which to find the file\n- `filename` - The name of the file to be copied from\n- `to_directory` - The relative directory within which to copy the file\n- `to_filename` - The name of the file to be copied to\n- `opts` - The options for the storage system\n  - `path:` (required) The base path within which to save files\n  - `acl:` (optional) The file mode to store the file (accepts octal `0o644` or string `\"0644\"`). See `File.chmod/2` for more info.","ref":"Trunk.Storage.Filesystem.html#copy/5","title":"Trunk.Storage.Filesystem.copy/5","type":"function"},{"doc":"The file will be copied from /opts/uploads/path/to/file.ext to /opts/uploads/copied/to/file.copy\n```\nTrunk.Storage.Filesystem.copy(\"path/to/\", \"file.ext\", \"copied/to/\", \"file.copy\", path: \"/opt/uploads\")","ref":"Trunk.Storage.Filesystem.html#copy/5-example","title":"Example: - Trunk.Storage.Filesystem.copy/5","type":"function"},{"doc":"Deletes the file from the file system.\n\n- `directory` - The relative directory within which to store the file\n- `filename` - The name of the file to be saved\n- `opts` - The options for the storage system\n  - `path:` (required) The base path within which to save files","ref":"Trunk.Storage.Filesystem.html#delete/3","title":"Trunk.Storage.Filesystem.delete/3","type":"function"},{"doc":"The file will be removed from /opts/uploads/path/to/file.ext\n```\nTrunk.Storage.Filesystem.delete(\"path/to/\", \"file.ext\", path: \"/opt/uploads\")","ref":"Trunk.Storage.Filesystem.html#delete/3-example","title":"Example: - Trunk.Storage.Filesystem.delete/3","type":"function"},{"doc":"","ref":"Trunk.Storage.Filesystem.html#retrieve/4","title":"Trunk.Storage.Filesystem.retrieve/4","type":"function"},{"doc":"Saves the file to the file system.\n\n- `directory` - The relative directory within which to store the file\n- `filename` - The name of the file to be saved\n- `source_path` - The full path to the file to be stored. This is a path to the uploaded file or a temporary file that has undergone transformation\n- `opts` - The options for the storage system\n  - `path:` (required) The base path within which to save files\n  - `acl:` (optional) The file mode to store the file (accepts octal `0o644` or string `\"0644\"`). See `File.chmod/2` for more info.","ref":"Trunk.Storage.Filesystem.html#save/4","title":"Trunk.Storage.Filesystem.save/4","type":"function"},{"doc":"The file will be saved to /opts/uploads/path/to/file.ext\n```\nTrunk.Storage.Filesystem.save(\"path/to/\", \"file.ext\", \"/tmp/uploaded_file.ext\", path: \"/opt/uploads\")","ref":"Trunk.Storage.Filesystem.html#save/4-example","title":"Example: - Trunk.Storage.Filesystem.save/4","type":"function"},{"doc":"A `Trunk.Storage` implementation for Amazon’s S3 service.","ref":"Trunk.Storage.S3.html","title":"Trunk.Storage.S3","type":"module"},{"doc":"Generates a URL to the S3 object\n\n- `directory` - The directory (will be combined with the `filename` to form the S3 object key.\n- `filename` - The name of the file (will be combined with the `directory` to form the S3 object key.\n- `opts` - The options for the storage system\n  - `bucket:` (required) The S3 bucket in which to store the object.\n  - `virtual_host:` (optional) boolean indicator whether to generate a virtual host style URL or not.\n  - `signed:` (optional) boolean whether to sign the URL or not.\n  - `ex_aws:` (optional) override options for `ex_aws`","ref":"Trunk.Storage.S3.html#build_uri/3","title":"Trunk.Storage.S3.build_uri/3","type":"function"},{"doc":"```\nTrunk.Storage.S3.build_url(\"path/to\", \"file.ext\", bucket: \"my-bucket\")\n#=> \"https://s3.amazonaws.com/my-bucket/path/to/file.ext\"\nTrunk.Storage.S3.build_url(\"path/to\", \"file.ext\", bucket: \"my-bucket\", virtual_host: true)\n#=> \"https://my-bucket.s3.amazonaws.com/path/to/file.ext\"\nTrunk.Storage.S3.build_url(\"path/to\", \"file.ext\", bucket: \"my-bucket\", signed: true)\n#=> \"https://s3.amazonaws.com/my-bucket/path/to/file.ext?X-Amz-Algorithm=AWS4-HMAC-SHA256&…\"\n```","ref":"Trunk.Storage.S3.html#build_uri/3-example","title":"Example: - Trunk.Storage.S3.build_uri/3","type":"function"},{"doc":"Copies the file within Amazon S3.\n\n- `directory` - The relative directory within which to find the file (will be combined with the `filename` to form the source S3 object key).\n- `filename` - The name of the file to be copied from (will be combined with the `directory` to form the source S3 object key).\n- `to_directory` - The relative directory within which to copy the file (will be combined with the `to_filename` to form the destination S3 object key).\n- `to_filename` - The name of the file to be copied to (will be combined with the `to_directory` to form the destination S3 object key).\n- `opts` - The options for the storage system\n  - `path:` (required) The base path within which to save files\n  - `acl:` (optional) The file mode to store the file (accepts octal `0o644` or string `\"0644\"`). See `File.chmod/2` for more info.","ref":"Trunk.Storage.S3.html#copy/5","title":"Trunk.Storage.S3.copy/5","type":"function"},{"doc":"The file will be copied from s3.amazonaws.com/my-bucket/path/to/file.ext to s3.amazonaws.com/my-bucket/copied/to/file.copy\n```\nTrunk.Storage.S3.copy(\"path/to/\", \"file.ext\", \"copied/to/\", \"file.copy\", bucket: \"my-bucket\")","ref":"Trunk.Storage.S3.html#copy/5-example","title":"Example: - Trunk.Storage.S3.copy/5","type":"function"},{"doc":"Deletes the file from Amazon S3.\n\n- `directory` - The directory (will be combined with the `filename` to form the S3 object key.\n- `filename` - The name of the file (will be combined with the `directory` to form the S3 object key.\n- `opts` - The options for the storage system\n  - `bucket:` (required) The S3 bucket in which to store the object\n  - `ex_aws:` (optional) override options for `ex_aws`","ref":"Trunk.Storage.S3.html#delete/3","title":"Trunk.Storage.S3.delete/3","type":"function"},{"doc":"The file will be removed from s3.amazonaws.com/my-bucket/path/to/file.ext\n```\nTrunk.Storage.S3.delete(\"path/to/\", \"file.ext\", bucket: \"my-bucket\")","ref":"Trunk.Storage.S3.html#delete/3-example","title":"Example: - Trunk.Storage.S3.delete/3","type":"function"},{"doc":"","ref":"Trunk.Storage.S3.html#retrieve/4","title":"Trunk.Storage.S3.retrieve/4","type":"function"},{"doc":"Saves the file to Amazon S3.\n\n- `directory` - The directory (will be combined with the `filename` to form the S3 object key).\n- `filename` - The name of the file (will be combined with the `directory` to form the S3 object key).\n- `source_path` - The full path to the file to be stored. This is a path to the uploaded file or a temporary file that has undergone transformation\n- `opts` - The options for the storage system\n  - `bucket:` (required) The S3 bucket in which to store the object\n  - `ex_aws:` (optional) override options for `ex_aws`\n  - All other options are passed through to S3 put object which means you can pass in anything accepted by `t:ExAws.S3.put_object_opts/0` including but not limited to `:acl`, `:meta`, `:content_type`, and `:content_disposition`","ref":"Trunk.Storage.S3.html#save/4","title":"Trunk.Storage.S3.save/4","type":"function"},{"doc":"The file will be saved to s3.amazonaws.com/my-bucket/path/to/file.ext\n```\nTrunk.Storage.S3.save(\"path/to/\", \"file.ext\", \"/tmp/uploaded_file.ext\", bucket: \"my-bucket\")","ref":"Trunk.Storage.S3.html#save/4-example","title":"Example: - Trunk.Storage.S3.save/4","type":"function"},{"doc":"This module defines a `Trunk.VersionState` struct and provides some helper functions for working with that state.\n\nMost of these fields are used internally during processing.","ref":"Trunk.VersionState.html","title":"Trunk.VersionState","type":"module"},{"doc":"The following fields are available in the version state object. Some values are filled in during processing.\n- `temp_path` - The path to the temporary file created for transformation. If the version doesn't undergo transformation, no temporary path will be available.\n- `transform` - The transform instruction returned from `c:Trunk.transform/2`\n- `storage_dir` - The storage directory returned from `c:Trunk.storage_dir/2`\n- `filename` - The filename returned from `c:Trunk.filename/2`\n- `storage_opts` - The additional storage options returned from `c:Trunk.storage_opts/2`\n- `assigns` - shared user data as a map (Same as assigns in  `Plug.Conn`)","ref":"Trunk.VersionState.html#module-fields","title":"Fields - Trunk.VersionState","type":"module"},{"doc":"This information is made available during `c:Trunk.postprocess/3` which is called once the transformation is complete but before the storage callbacks are called. At this point you can work with the transformed version file and assign data that can be used later when determining the storage directory, filename and storage options.","ref":"Trunk.VersionState.html#module-usage","title":"Usage - Trunk.VersionState","type":"module"},{"doc":"Assigns a value to a key on the state.","ref":"Trunk.VersionState.html#assign/3","title":"Trunk.VersionState.assign/3","type":"function"},{"doc":"```\niex> version_state.assigns[:hello]\nnil\niex> version_state = assign(version_state, :hello, :world)\niex> version_state.assigns[:hello]\n:world\n```","ref":"Trunk.VersionState.html#assign/3-example","title":"Example: - Trunk.VersionState.assign/3","type":"function"},{"doc":"","ref":"Trunk.VersionState.html#t:t/0","title":"Trunk.VersionState.t/0","type":"type"},{"doc":"# MIT License\n\nCopyright (c) 2017 Andrew Timberlake\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html","title":"License","type":"extras"},{"doc":"# Trunk\n\n[![Build Status](https://circleci.com/gh/andrewtimberlake/trunk.svg?style=shield)](https://circleci.com/gh/andrewtimberlake/trunk)\n[![Module Version](https://img.shields.io/hexpm/v/trunk.svg)](https://hex.pm/packages/trunk)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/trunk/)\n[![Total Download](https://img.shields.io/hexpm/dt/trunk.svg)](https://hex.pm/packages/trunk)\n[![License](https://img.shields.io/hexpm/l/trunk.svg)](https://github.com/andrewtimberlake/trunk/blob/master/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/andrewtimberlake/trunk.svg)](https://github.com/andrewtimberlake/trunk/commits/master)\n\nA file attachment/storage library for Elixir.","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"Add `:trunk` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:trunk, \"~> 1.0\"},\n\n    # If you want to use Amazon S3, then add:\n    {:ex_aws_s3, \"~> 2.0\"},\n    {:hackney,   \"~> 1.7\"},\n    {:poison,    \"~> 3.1\"},\n    {:sweet_xml, \"~> 0.6\"},\n  ]\nend\n```\n\nTrunk has only one _hard_ dependency on [Briefly](https://hex.pm/packages/briefly) to handle temporary file creation (and auto-destruction)","ref":"readme.html#installation","title":"Installation - Overview","type":"extras"},{"doc":"Trunk is a behaviour that is implemented in a module. `use Trunk` creates functions for storing, deleting, and generating urls for your files and their versions. It then implements callbacks in the simplest way possible. You can then override the callbacks you need to as you want to extend the behaviour.\n\n```elixir\ndefmodule MyTrunk do\n  use Trunk, versions: [:original, :thumb]\n\n  # override callbacks as needed.\nend\n```","ref":"readme.html#usage","title":"Usage - Overview","type":"extras"},{"doc":"Trunk has been designed to be highly configurable. It can be configured in stages with each level merging with the level before it.\n\nSee the [documentation](https://hexdocs.pm/trunk/Trunk.html#module-options) for all config options.\n\n#","ref":"readme.html#configuration","title":"Configuration - Overview","type":"extras"},{"doc":"```elixir\nconfig :trunk,\n  storage: Trunk.Storage.Filesystem,\n  storage_opts: [path: \"/tmp\"]\n```\n\n#","ref":"readme.html#global-configuration","title":"Global configuration - Overview","type":"extras"},{"doc":"```elixir\nconfig :my_app, :trunk,\n  storage: Trunk.Storage.S3,\n  storage_opts: [bucket: \"test-trunk\"]\n```\n\nin order for these options to be used, you need to pass the `otp_app` option when calling `use Trunk` as follows:\n\n```elixir\ndefmodule MyTrunk do\n  use Trunk, otp_app: :my_app\nend\n```\n\n#","ref":"readme.html#app-specific-configuration-for-umbrella-type-configs","title":"App specific configuration for umbrella type configs - Overview","type":"extras"},{"doc":"```elixir\ndefmodule MyTrunk do\n  use Trunk, versions: [:original, :trunk],\n             storage: Trunk.Storage.Filesystem,\n             storage_opts: [path: \"/tmp\"]\nend\n```\n\n#","ref":"readme.html#module-configuration","title":"Module configuration - Overview","type":"extras"},{"doc":"**Caution:** If you override options during the storage call, you need to be sure to pass the same options to other calls\n\n```elixir\nMyTrunk.store(\"/path/to/file.ext\", storage: Trunk.Storage.S3, storage_opts: [bucket: \"test-trunk\"])\n```","ref":"readme.html#function-options","title":"Function options - Overview","type":"extras"},{"doc":"Storage is handled by a behaviour.\n\nTwo storage handlers are included: [Trunk.Storage.Filesystem](https://hexdocs.pm/trunk/Trunk.Storage.Filesystem.html), and [Trunk.Storage.S3](https://hexdocs.pm/trunk/Trunk.Storage.S3.html)\nAdditional storage systems can be handled by creating a module that implements the [Trunk.Storage](https://hexdocs.pm/trunk/Trunk.Storage.html) behaviour.\n\nWhen storing files, Trunk runs the file through a transformation pipeline allowing you to generate different versions of a file.\nEach stage in the pipeline is handled via a callback allowing you to configure what transformations take place, where the version is stored and how it is named.\nFull information can be found [in the documentation](https://hexdocs.pm/trunk/Trunk.Storage.html#content)","ref":"readme.html#storage","title":"Storage - Overview","type":"extras"},{"doc":"You have the option of passing a scope (usually a struct or map) into the transform functions. This scope object will then be available in each callback allowing you to further customise the handling of each version.\n\n#","ref":"readme.html#scope","title":"Scope - Overview","type":"extras"},{"doc":"```elixir\ndefmodule MyTrunk do\n  use Trunk, versions: [:thumb]\n\n  def storage_dir(%Trunk.State{scope: %{id: model_id}}, :thumb)\n    do: \"my_models/#{model_id}\"\nend\n\nMyTrunk.store(\"/path/to/file.ext\", %{id: 42})\n# will save to  /my_models/42/ \n```","ref":"readme.html#example","title":"Example: - Overview","type":"extras"},{"doc":"State about the file and the version transformations is kept in a [Trunk.State](https://hexdocs.pm/trunk/Trunk.State.html) struct which is passed to each callback. Each version also keeps track of its own state in a [Trunk.VersionState](https://hexdocs.pm/trunk/Trunk.VersionState.html) struct which is also available through the `Trunk.State.versions` map.\n\n#","ref":"readme.html#state","title":"State - Overview","type":"extras"},{"doc":"One of the key features of Trunk is the ability to take a file and produce transformed versions. Perhaps you want to take an uploaded photo and produce a thumbnail, take a video and extract thumbnails, or take an XLS file and produce a CSV file for easier processing.\nThis is all handled easily with the flexible version and transform system.\n\nSee full documentation on [Trunk.transform/2](https://hexdocs.pm/trunk/Trunk.html#c:transform/2)","ref":"readme.html#transformation","title":"Transformation - Overview","type":"extras"},{"doc":"A shout out to [stavro](https://github.com/stavro) who created [arc](https://github.com/stavro/arc) which I used in many of my projects, and which provided much inspiration for what resulted in Trunk.\n\n#","ref":"readme.html#credits","title":"Credits - Overview","type":"extras"},{"doc":"- [coffee.jpg](https://unsplash.com/photos/Cdz_lvnl37k) by [Ronaldo Arthur Vidal](https://unsplash.com/@ronaldoav)\n- [coffee beans.jpg](http://unsplash.com/photos/JS-QXqSGVE8) by [Alex Jones](https://unsplash.com/@alexjones)","ref":"readme.html#photos-used-in-testing","title":"Photos used in testing - Overview","type":"extras"},{"doc":"Copyright (c) 2017 Andrew Timberlake\n\nThis work is free. You can redistribute it and/or modify it under the\nterms of the MIT License. See the [LICENSE.md](./LICENSE.md) file for more details.","ref":"readme.html#copyright-and-license","title":"Copyright and License - Overview","type":"extras"},{"doc":"# Examples","ref":"usage.html","title":"Usage","type":"extras"},{"doc":"This example does nothing but implement Trunk and override the `c:Trunk.transform/2` callback for the `:thumbnail` version\n\n```elixir\ndefmodule MyTrunk do\n  use Trunk, versions: [:original, :thumbnail]\n\n  def transform(_state, :thumbnail),\n    do: {:convert, \"-strip -thumbnail 100x100>\"}\n  def transform(state, version), do: super(state, version)\nend\n\n{:ok, _state} = MyTrunk.store(\"/path/to/picture.jpg\")\n# Will store original in  /picture.jpg\n#        and thumbnail in  /picture_thumbnail.jpg\n```","ref":"usage.html#simple-example","title":"Simple Example - Usage","type":"extras"},{"doc":"This example simply shows a full implementation with all the callback functions overridden.\n\n```elixir\ndefmodule MyTrunk do\n  use Trunk, versions: [:original, :thumbnail],\n             storage: Trunk.Storage.S3,\n             storage_opts: [bucket: \"my-bucket\",\n                            ex_aws: [acl: :public_read]]\n\n  def preprocess(%Trunk.State{lower_extname: extname} = state) do\n    if extname in [\".png\", \".jpg\", \".jpeg\"] do\n      {:ok, state}\n    else\n      {:error, \"Invalid file\"}\n    end\n  end\n\n  def transform(_state, :thumbnail),\n    do: {:convert, \"-strip -thumbnail 100x100>\", :jpg}\n  def transform(_state, :original), do: nil\n\n  def postprocess(%Trunk.VersionState{temp_path: temp_path} = version_state, :thumbnail, _state) do\n    hash = :crypto.hash(:md5, File.read!(temp_path)) |> Base.encode16(case: :lower)\n    {:ok, Trunk.VersionState.assign(version_state, :hash, hash)}\n  end\n  def postprocess(version_state, version, state), do: super(version_state, version, state)\n\n  def storage_opts(_state, :original),\n    do: [acl: :private]\n  def storage_opts(_state, :thumbnail),\n    do: [acl: :private, content_type: \"image/jpeg\"]\n\n  def storage_dir(%Trunk.State{scope: %MyModel{id: model_id}}, _version),\n    do: \"photos/#{model_id}\"\n\n  def filename(state, :thumbnail),\n    do: \"thumbnail_#{Trunk.State.get_version_assign(state, :thumbnail, :hash)}.jpg\"\n  def filename(%Trunk.State{filename: filename}, :original), do: filename\nend\n\niex> my_model = %MyModel{id: 42}\niex> {:ok, state} = MyTrunk.store(\"/path/to/picture.png\", my_model)\n# Will store original in s3://my-bucket/photos/42/picture.png\n#        and thumbnail in s3://my-bucket/photos/42/thumbnail_889c00fed0f5382b4bdea612ae7a42df.jpg\n\niex> %{my_model | file: Trunk.State.save(state, as: :map)}\n%MyModel{id: 42, file: %{filename: \"picture.png\", version_assigns: %{thumbnail: %{hash: \"889c00fed0f5382b4bdea612ae7a42df\"}}}}\n\niex> {:error, \"Invalid file\"} = MyTrunk.store(\"/path/to/document.pdf\", my_model)\n```","ref":"usage.html#full-example","title":"Full Example - Usage","type":"extras"},{"doc":"# Changelog","ref":"changelog.html","title":"Change Log","type":"extras"},{"doc":"- Added CHANGELOG\n- Add `Trunk.copy/4` to copy files between \"folders\" within the same file system (different directores in filesystem or different object keys in same S3 bucket)","ref":"changelog.html#1-2-0","title":"1.2.0 - Change Log","type":"extras"},{"doc":"- Return `nil` from `Trunk.filename/2` to prevent storing a specific version","ref":"changelog.html#1-2-1","title":"1.2.1 - Change Log","type":"extras"},{"doc":"- Returns `nil` from `url/1,2,3,4` if the `Trunk.filename/2` returns `nil`","ref":"changelog.html#1-3-0","title":"1.3.0 - Change Log","type":"extras"}]}